// Example16View.cpp : implementation of the CExample16View class
//

#include "stdafx.h"
#include "Example16.h"

#include "mainfrm.h"

#include "Example16Doc.h"
#include "Example16View.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CExample16View

IMPLEMENT_DYNCREATE(CExample16View, CView)

BEGIN_MESSAGE_MAP(CExample16View, CView)
	//{{AFX_MSG_MAP(CExample16View)
	ON_WM_CREATE()
	ON_WM_KEYDOWN()
	ON_WM_VSCROLL()
	ON_COMMAND(ID_VIEW_FONT, OnViewFont)
	ON_WM_SIZE()
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CExample16View construction/destruction

CExample16View::CExample16View()
{
	m_pFont = NULL;
	m_pPrintFont = NULL;
	m_bNoFont = TRUE;

	m_bInsurance = FALSE;
	
	CWindowDC dc(NULL);
	int cyPixels = dc.GetDeviceCaps(LOGPIXELSY);
	
	memset (&m_lf, 0, sizeof(LOGFONT));
	lstrcpy (m_lf.lfFaceName, _T("Fixedsys"));	   // Default font

	m_lf.lfHeight = (-1) * MulDiv(10, cyPixels, 72);

	m_lf.lfWeight = 400;
	m_lf.lfOutPrecision = OUT_CHARACTER_PRECIS;
	m_lf.lfClipPrecision = CLIP_STROKE_PRECIS;
	m_lf.lfQuality = DRAFT_QUALITY;
	m_lf.lfPitchAndFamily = FIXED_PITCH|FF_MODERN;

	fontpoint = 10;
		
	m_bTabs = TRUE;
	m_pnTabs = 0;

	// Print
	m_nLinesPerPrintPage = 0;
	m_nPrintPages = 0;

	topline = 0;

	m_iCurrLn = 1;
	m_iCurrCol = 1;
}

CExample16View::~CExample16View()
{				
	if (m_pFont)
		{
		delete m_pFont;
		m_pFont = 0;

		if (m_pnTabs)
			delete []m_pnTabs;
		}
}

BOOL CExample16View::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CView::PreCreateWindow(cs);
}

CMainFrame * CExample16View::GetFrame()
{
	CMainFrame * pFrame = (CMainFrame *)AfxGetApp()->m_pMainWnd;
	ASSERT(pFrame->IsKindOf(RUNTIME_CLASS(CMainFrame)));
	return pFrame;
}

/////////////////////////////////////////////////////////////////////////////
// CExample16View drawing

void CExample16View::OnDraw(CDC* pDC)
{
	BOOL	bFlag = TRUE;
	BOOL	isprinting = pDC->IsPrinting();
	
	CExample16Doc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// Select current font into DC.

	if (isprinting)
		pDC->SelectObject(m_pPrintFont);
	else
		pDC->SelectObject(m_pFont);

	CMainFrame *pFrame = GetFrame();

	// Select text color.

	pDC->SetTextColor(pFrame->m_crForeground);

	// Select text background color.

	pDC->SetBkColor(pFrame->m_crBackground);

	// Figure out which lines to draw.
	CStringList *pLineList = GetDocument()->GetLineList();
	
	int iLine;
	int cLastLine;
	int cyLine = 0;
	int	iCount = 0;
	
	if (isprinting)
		{
		iLine = prntLine;
		cLastLine = prntLastLine + 1;
		cyLine = -2 * (m_cyLineHeight);
		}
	else
		{
		iLine = m_iTopLine;
		cLastLine = m_iTopLine + m_clHeight + 1;    
		}
		
	cLastLine = min(cLastLine, pLineList->GetCount());

	// Loop through client area coordinates to draw.
	
	CString		strLine;
	POSITION	pos;
	
	while (iLine < cLastLine)
		{

		// Draw a line of text.
		if((pos = pLineList->FindIndex(iLine)) != NULL)
			{
			strLine = pLineList->GetAt(pos);

			pDC->TextOut (m_cxLeftMargin, cyLine, strLine);
			
			if (isprinting)
				{
				cyLine -= m_cyLineHeight;
				}
			else
				cyLine += m_cyLineHeight;

			iLine++;
			}
		}
}

/////////////////////////////////////////////////////////////////////////////
// CExample16View printing

BOOL CExample16View::OnPreparePrinting(CPrintInfo* pInfo)
{
	ComputePrintMetrics(pInfo);

	if (pInfo->m_bPreview)
		{
		pInfo->m_nNumPreviewPages = m_nPrintPages;
		pInfo->SetMaxPage(m_nPrintPages);
		}

	pInfo->m_pPD->m_pd.Flags |= PD_DISABLEPRINTTOFILE;
	
	return CView::DoPreparePrinting(pInfo);
}

void CExample16View::OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo)
{
	old_height = m_cyLineHeight;
	old_margin = m_cxLeftMargin;

	old_topline = m_iTopLine;

	SetupPrinterFont(pDC, pInfo);
}

void CExample16View::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	m_cyLineHeight = old_height;
	m_cxLeftMargin = old_margin;

	m_iTopLine = old_topline;

	delete m_pPrintFont;
}

/////////////////////////////////////////////////////////////////////////////
// CExample16View diagnostics

#ifdef _DEBUG
void CExample16View::AssertValid() const
{
	CView::AssertValid();
}

void CExample16View::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CExample16Doc* CExample16View::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CExample16Doc)));
	return (CExample16Doc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CExample16View message handlers

void CExample16View::ComputeViewMetrics()
{
	CDC*	pDC = CDC::FromHandle(::GetDC(NULL));
	int nSaveDC = pDC->SaveDC();

	pDC->SetMapMode(MM_LOENGLISH);

	CFont* 	pPreviousFont = pDC->SelectObject(GetFont());

	// Get size of logical inch in pixels:
	m_sizeLogInch.cx = pDC->GetDeviceCaps(LOGPIXELSX);
	m_sizeLogInch.cy = pDC->GetDeviceCaps(LOGPIXELSY);

	TEXTMETRIC tm;
	pDC->GetTextMetrics(&tm);

	// Calculate view character size
	m_ViewCharSize.cy = tm.tmHeight + tm.tmExternalLeading;
	m_ViewCharSize.cx = tm.tmAveCharWidth;

	// Convert to device units to minimize round off error
	pDC->LPtoDP(&m_ViewCharSize);
	
	// Calculate document size
	CExample16Doc* pDoc = GetDocument();
	m_DocSize.cy = m_ViewCharSize.cy * pDoc->GetLineList()->GetCount();

	// loop through the document and find the longest line
	CString szLine;
	CSize	size;

	POSITION pos = pDoc->GetLineList()->GetHeadPosition();

	while (pos != NULL)
		{
		szLine = pDoc->GetLineList()->GetNext(pos);
		size = pDC->GetTextExtent(szLine, szLine.GetLength());
		m_DocSize.cx = max(size.cx, m_DocSize.cx);
		}

	// Account for our margin
	m_DocSize.cx +=	4 * m_ViewCharSize.cx;
	
	// Clean up
	if (pPreviousFont)
		{
		pDC->SelectObject(pPreviousFont);
		}

	pDC->RestoreDC(nSaveDC);
	::ReleaseDC(NULL, pDC->GetSafeHdc());
}

BOOL CExample16View::CreateNewFont()
{
	// Delete any previous font.
	if (m_pFont)
		delete m_pFont;

	m_pFont = new CFont();
	if (!m_pFont)
		return FALSE;

	m_pFont->CreateFontIndirect(&m_lf);

	// Calculate font height.
	CClientDC	dc(this);
	TEXTMETRIC	tm;
	dc.SelectObject(m_pFont);
	dc.GetTextMetrics(&tm);
	m_cyLineHeight = tm.tmHeight + tm.tmExternalLeading;

	// Calculate left margin.
	m_cxLeftMargin = tm.tmAveCharWidth * 2;

	// Rebuild tab setting table.
	if (m_pnTabs)
		delete [] m_pnTabs;

	// Recalculate scroll bar info.
	ResetScrollValues();

	GetDocument()->UpdateAllViews(NULL);

	return TRUE;
}

void CExample16View::ResetScrollValues()
{
	// Set count of lines in window height

	if (!m_bInsurance)
		{
		RECT	rect;

		GetClientRect(&rect);
		m_cyClient = rect.bottom;
		}

	m_clHeight = m_cyClient/m_cyLineHeight;

	CStringList *pLineList = GetDocument()->GetLineList();

	if (pLineList->GetCount() <= m_clHeight)
		{
		SetScrollRange(SB_VERT, 0, 0, TRUE);
		m_iTopLine = 0;
		return;
		}

	// Adjust scroll range for new window size.

	SetScrollRange (SB_VERT, 0, pLineList->GetCount() - m_clHeight, FALSE);

	// Adjust scroll thumb position.

	SetScrollPos(SB_VERT, m_iTopLine, TRUE);
}

CFont* CExample16View::GetFont()	 
{
	if (m_pFont == NULL)
		{
		m_pFont = new CFont;
		if (m_pFont)
			{
			m_pFont->CreatePointFont(100, "Fixedsys");
			m_bNoFont = FALSE;
			}
		}
	return m_pFont;
}

int CExample16View::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	if (!CreateNewFont())
		return -1;
	
	return 0;
}

void CExample16View::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	CExample16Doc* pDoc = GetDocument();

	if (pDoc->m_dataflag)
		{
		switch (nChar)
			{
			case VK_HOME:
				// Ctr-Home - go to top of doc
				if (::GetKeyState(VK_CONTROL) < 0)
					{
					OnVScroll(SB_TOP, 0, 0);
					}
				break;

			case VK_END:
				if (::GetKeyState(VK_CONTROL) < 0)
					{
					OnVScroll(SB_BOTTOM, 0, 0);
					}
				break;

			case VK_UP:
			case VK_LEFT:
				OnVScroll(SB_LINEUP, 0, 0);
				break;
			case VK_DOWN:
			case VK_RIGHT:
				OnVScroll(SB_LINEDOWN, 0, 0);
				break;
			case VK_PRIOR:
				OnVScroll(SB_PAGEUP, 0, 0);
				break;
			case VK_NEXT:
				OnVScroll(SB_PAGEDOWN, 0, 0);
				break;

			case VK_SNAPSHOT:
				return;
			}
		}
	
	CView::OnKeyDown(nChar, nRepCnt, nFlags);
}

void CExample16View::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	// Temporary "new line" value.

	int iTop = m_iTopLine;

	// Temporary copy of doc line count.
	CStringList *pLineList = GetDocument()->GetLineList();
	int cLines = pLineList->GetCount();

	// Based on particular scroll button clicked, modify top line index.

	switch (nSBCode)
		{
		case SB_BOTTOM:
			iTop = cLines - m_clHeight;
            break;
        case SB_ENDSCROLL:
            break;
        case SB_LINEDOWN:
            iTop++;
            break;
        case SB_LINEUP:
            iTop--;
            break;
        case SB_PAGEDOWN:
            iTop += m_clHeight;
            break;
        case SB_PAGEUP:
            iTop -= m_clHeight;
            break;
        case SB_THUMBPOSITION:
            iTop = nPos;
            break;
        case SB_THUMBTRACK:
            iTop = nPos;
            break;
        case SB_TOP:
            iTop = 0;
            break;
    }

    // Check range of new index;
    iTop = max (iTop, 0);
    iTop = min (iTop, cLines - m_clHeight);

    // If no change, ignore.
    if (iTop == m_iTopLine) return;

    // Pixels to scroll = (lines to scroll) * height-per-line.
    int cyScroll = (m_iTopLine - iTop) * m_cyLineHeight;

    // Define new top-line value.
     m_iTopLine = iTop;

    // Scroll pixels.
    ScrollWindow(0, cyScroll);

    // Adjust scroll thumb position.
    SetScrollPos(SB_VERT, m_iTopLine, TRUE);

	m_iCurrLn = m_iTopLine + 1;
	
	CView::OnVScroll(nSBCode, nPos, pScrollBar);
}

void CExample16View::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint) 
{
	int iTemp = m_iTopLine;

	CExample16Doc* pDoc = GetDocument();

	ComputeViewMetrics();

	// Reset "Index to top line displayed" = top of file.
	m_iTopLine = 0;
	// topline = 0;
		
	// Recalculate scroll bar info.
	ResetScrollValues();
		
	m_iCurrLn = 1;
	m_iCurrCol = 1;

	// Force repaint.
    Invalidate();		
}

void CExample16View::OnViewFont() 
{
	CDC printerDC;

	// Get access to a printer DC from the ini file ininstalled printer
	// GetPrinterDeviceDefaults(pPrintDlg);
	if (AfxGetApp()->CreatePrinterDC(printerDC))
		m_pSetFont = new CFontDialog(&m_lf, CF_SCREENFONTS | CF_PRINTERFONTS | 
			CF_FORCEFONTEXIST | CF_EFFECTS,	&printerDC, this);
	else
		m_pSetFont = new CFontDialog(&m_lf, CF_SCREENFONTS | CF_PRINTERFONTS | 
			CF_FORCEFONTEXIST | CF_EFFECTS,	NULL, this);
	
	if (!m_pSetFont)
		return;

	int	iRet = m_pSetFont->DoModal();

	if (iRet != IDOK)
		return;

	fontpoint = m_pSetFont->m_cf.iPointSize/10;

	CreateNewFont();

	ResetScrollValues();

	GetDocument()->UpdateAllViews(NULL);

	delete m_pSetFont;	
}

void CExample16View::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);
	
	// Save client area height.

	m_bInsurance = TRUE;

	m_cyClient = cy;

	ResetScrollValues();	
}

void CExample16View::SetupPrinterFont(CDC *pDC, CPrintInfo* ptrinfo)
{
	LOGFONT	lf;

	m_pFont->GetLogFont(&lf);

	m_pPrintFont = new CFont ();
	
	m_pPrintFont->CreateFontIndirect(&lf);	 
}

void CExample16View::CalculateMargins(CDC* pDC)
{
	POINT pt;

	pDC->Escape(GETPRINTINGOFFSET, 0, NULL, &pt);
	m_rectMargins.left = pt.x;
	m_rectMargins.top = pt.y;

	pDC->Escape(GETPHYSPAGESIZE, 0, NULL, &pt);
	m_rectMargins.right = pt.x - pDC->GetDeviceCaps(HORZRES) - m_rectMargins.left;
	m_rectMargins.bottom = pt.y - pDC->GetDeviceCaps(VERTRES) - m_rectMargins.top;
}

void CExample16View::OnPrepareDC(CDC* ptrdc, CPrintInfo* ptrinfo) 
{
	BOOL	isprinting = ptrdc->IsPrinting();

	CView::OnPrepareDC (ptrdc, ptrinfo); // let base class get the DCs

	CStringList *pLineList = GetDocument()->GetLineList();

	int iLine;
	int cLastLine;

	if (isprinting)
		{
		 ptrdc->SetMapMode(MM_LOENGLISH);
		 ptrdc->SelectObject(m_pPrintFont);
		 ComputePrintableLines(ptrdc, ptrinfo, iLine, cLastLine);
		 }
	else
		 {
		 iLine = m_iTopLine;
		 cLastLine = m_iTopLine + m_clHeight + 1;
		 }

	cLastLine = min(cLastLine, pLineList->GetCount());

	if (isprinting)
		{
		prntLine = iLine;
		prntLastLine = cLastLine;
		}

	if (ptrinfo != NULL) 
		{               
		// is printer this time
		// set the continue printing flag if there are more lines to print

		if (iLine < pLineList->GetCount()) 
			ptrinfo->m_bContinuePrinting = TRUE;
		else 
			ptrinfo->m_bContinuePrinting = FALSE;
		}
}

void CExample16View::ComputePrintableLines(CDC* pDC, CPrintInfo* pInfo, int& nFirst, int& nLast)
{
	CExample16Doc* pDoc = GetDocument();

	nFirst = GetLinesPerPrintPage() * (pInfo->m_nCurPage - 1);
	nLast = min(nFirst + GetLinesPerPrintPage() - 1,
					pDoc->GetLineList()->GetCount() - 1);
}

void CExample16View::ComputePrintMetrics(CPrintInfo* pInfo)
{
	CExample16Doc* pDoc = GetDocument();
	
	CDC* pDC = CreatePrinterDC();

	SetupPrinterFont(pDC, pInfo);

	CFont* pPreviousFont = pDC->SelectObject(m_pPrintFont);

	TEXTMETRIC tm;

	pDC->GetTextMetrics(&tm);

	pDC->SetMapMode(MM_LOENGLISH);

	m_PrintCharSize.cy = tm.tmHeight + tm.tmExternalLeading;
	m_PrintCharSize.cx = tm.tmAveCharWidth;

	CalculateMargins(pDC);

	Escape(pDC->m_hDC, GETPHYSPAGESIZE, NULL, NULL, &m_sizePhysDev);

	pDC->DPtoLP(&m_rectMargins);

	pDC->DPtoLP(&m_sizePhysDev);

	int cyPage = m_sizePhysDev.cy;

	cyPage -= (abs(m_rectMargins.top) + abs(m_rectMargins.bottom));

	m_nLinesPerPrintPage = abs(cyPage/(m_PrintCharSize.cy)) - 4;

	m_nPrintPages = pDoc->GetLineList()->GetCount()/m_nLinesPerPrintPage;

	if (pDoc->GetLineList()->GetCount() % m_nLinesPerPrintPage)
		{
		m_nPrintPages++;
		}

	if (pPreviousFont)	
		{
		pDC->SelectObject(pPreviousFont);
		}
	
	if (m_pPrintFont)
		delete m_pPrintFont;
	
	::DeleteDC(pDC->GetSafeHdc());
}

CDC* CExample16View::CreatePrinterDC()
{
	ASSERT_VALID(this);
	PRINTDLG	PrtDlg;
	HDC			hDC;

	if (!AfxGetApp()->GetPrinterDeviceDefaults(&PrtDlg))
		{
		hDC = ::CreateDC("display", NULL, NULL, NULL);
		}
	else
		{
		CPrintDialog dlg(FALSE);
		dlg.m_pd.hDevMode = PrtDlg.hDevMode;
		dlg.m_pd.hDevNames = PrtDlg.hDevNames;
		hDC = dlg.CreatePrinterDC();
		}

	CDC* pDC = CDC::FromHandle(hDC);
	pDC->m_bPrinting = TRUE;
	return pDC;
}